package cn.itheima;


import java.util.ArrayList;

public class LoopList<T> {

	/**
	 * 编写一个泛型类，循环队列 LoopList<E> ，在构造方法中设置队列的大小 ，
	 * 队列中只存储最近添加的N个对象，后添加的对象会将先添加的对象挤出队列。
	 * 这个类提供 add(Object obj) 、  getFirst() 、 getChildAt(int n) 、 getSize() 等方法。
		
     * 说明：如wps等应用打开文件时，会保存最近n次打开文件的记录。
	 * 代码如：
	 * LoopList<String> myLoopList = new LoopList<String>(3);                //设置对列大小为 3
	 * myLoopList.add("one");        //添加第1个对象
	 * myLoopList.add("two");        //添加第2个对象
	 * myLoopList.add("three");        //添加第3个对象
	 * myLoopList.add("four");        //添加第4个对象、由于队列大小为3，所以，第4个对象会将第1个对象挤出去。
		
	 * syso(getfirst());                //打印结果为    "two"
	 * syso(getChildAt(1));        //打印结果为    "three"        
		
	 * myLoopList.add("five");        //继续添加会将当前的第1个对象挤出去。
		
	 * syso(getfirst());                //打印结果为    "three"
	 * syso(getChildAt(1));        //打印结果为    "four"
	 */
	private int quit;//队列元素出队指针
	private int size;//队列中元素个数
	
	LoopList(int size){
		this.size = size;
	}
	private ArrayList<T> queue = new ArrayList<T>(size);
	//获取元素个数
	public int getSize(){
		return queue.size();
	}
	
	//添加元素方法
	public void add(T element){
		if(this.getSize()==size)//如果队列的长度等于规定的长度
			delElement();       //那么删除第一个元素
		queue.add(element);		//在队列末尾添加元素
	}
	
	//删除第一个元素的方法
	public T delElement(){

		if(quit==size && size!=0){
			quit = 0;
		}
		return queue.remove(0);

	}
	
	//获取第一个元素
	public T getFirst(){
		return this.getChildAt(0);
	}
	
	//获取任意位置的元素
	public T getChildAt(int index){
		return queue.get(index);
	}
	
	public static void main(String[] args) {//主方法，测试泛型类
		LoopList<String> list = new LoopList<String>(3);
		
		list.add("one");
		list.add("two");
		list.add("three");
		list.add("four");
		
		sop(list.getFirst());
		sop(list.getChildAt(1));
		sop(list.getSize());

		list.add("five");
		list.add("six");
		list.add("seven");
		sop(list.getFirst());
		sop(list.getChildAt(2));
		sop(list.getSize());
	}

	public static void sop(Object obj){
		System.out.println(obj);
	}
}
